#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QGraphicsView>
#include <QMessageBox>
#include <QPrinter>
#include <QPageSetupDialog>
#include <QGeoServiceProvider>
#include <QGeoMappingManager>
#include <QPrintDialog>
#include <QFileDialog>
#include <QTranslator>
#include <plot.h>
#include <data.h>
#include <explorer.h>
#include <map.h>
#include <exportdialog.h>
#include <help.h>
#include <about.h>
#include <importhandler.h>
#include <thumbnailviewer.h>

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

    int load();

    /**Holds the thumbnails.*/
    QGraphicsScene scene;
    /**Enables the user to resize the explorer.*/
    QSplitter *splitter;
    /**Holds the plotters.*/
    plot *plotter[5];
    /**Printer handle for printing.*/
    QPrinter *printer;
    /**Explorer for browsing and loading data.*/
    explorer *exp;
    /**Exportdialog for exporting data.*/
    exportdialog *expdialog;
    /**Importhandle, handles import from file and from devices.*/
    importhandler *import;
    /**Holds the map.*/
    map *gpsmap;
    /**Manages the geomap.*/
    QGeoMappingManager *manager;
    /**Provides geo services.*/
    QGeoServiceProvider *geoprovider;
    /**Is set to true while loading data.*/
    bool loading;
    /**xdata, holds values from 0 to N, enables scaling by time.*/
    double *xdata;

    bool data_loaded;
    

public slots:
    void showPicture(QString fileName);
    void closePicture();
    void highlighted(QString fileName);

private slots:
    void on_actionBeenden_triggered();
    void on_actionAbout_triggered();
    void on_comboAltitudeColor_currentIndexChanged(int index);
    void on_comboAltitudeThickness_currentIndexChanged(int index);
    void on_checkAltitude_toggled(bool checked);
    void on_CoordResetButton_clicked();
    void on_actionDruckereinstellungen_triggered();
    void on_checkXSpeed_toggled(bool checked);
    void on_checkYSpeed_toggled(bool checked);
    void on_comboXSpeedColor_currentIndexChanged(int index);
    void on_comboYSpeedColor_currentIndexChanged(int index);
    void on_comboXSpeedThickness_currentIndexChanged(int index);
    void on_comboYSpeedThickness_currentIndexChanged(int index);
    void on_actionLaden_triggered();
    void on_actionCopy_triggered();
    void on_actionMerge_triggered();
    void on_actionImportieren_triggered();
    void on_actionDrucken_triggered();
    void slotLanguageChange(QAction* action);
    void on_actionL_schen_triggered();
    void on_checkXAcc_toggled(bool checked);
    void on_checkYAcc_toggled(bool checked);
    void on_checkZAcc_toggled(bool checked);
    void on_comboXAccColor_currentIndexChanged(int index);
    void on_comboYAccColor_currentIndexChanged(int index);
    void on_comboZAccColor_currentIndexChanged(int index);
    void on_comboXAccThickness_currentIndexChanged(int index);
    void on_comboYAccThickness_currentIndexChanged(int index);
    void on_comboZAccThickness_currentIndexChanged(int index);
    void on_checkXAngAcc_toggled(bool checked);
    void on_checkYAngAcc_toggled(bool checked);
    void on_checkZAngAcc_toggled(bool checked);
    void on_comboXAngAccColor_currentIndexChanged(int index);
    void on_comboYAngAccColor_currentIndexChanged(int index);
    void on_comboZAngAccColor_currentIndexChanged(int index);
    void on_comboXAngAccThickness_currentIndexChanged(int index);
    void on_comboYAngAccThickness_currentIndexChanged(int index);
    void on_comboZAngAccThickness_currentIndexChanged(int index);
    void on_comboOBDValue_currentIndexChanged(int index);
    void on_comboOBDColor_currentIndexChanged(int index);
    void on_comboOBDThickness_currentIndexChanged(int index);
    void loadConfig();
    void saveConfig();
    void createConfig();
    void restoreConfig();
    void on_listWidget_doubleClicked();
    void closeEvent(QCloseEvent *);
    void on_spinCoordBegin_valueChanged(int arg1);
    void on_spinCoordEnd_valueChanged(int arg1);
    void on_actionExport_triggered();
    void on_actionHilfe_triggered();
    void on_actionOptions_triggered();
    void on_pushButton_clicked(bool checked);
    void on_pushButton_2_clicked();

private:
    void updateView();
    void createLanguageMenu(void);
    bool eventFilter(QObject *obj, QEvent *event);
    /**Label to hold the pictures.*/
    QLabel *pictureLabel;
    /**The enlarged thumbnails are shown in this QDockWidget.*/
    QDockWidget *imageViewer;
    Ui::MainWindow *ui;
    /**Handles translation of all visible Strings when the language is changed.*/
    QTranslator translator;
    /**Displays the thumbnails of the pictures in the DCIM folder of the data set.*/
    thumbnailviewer *thumbviewer;
};

#endif // MAINWINDOW_H
