#ifndef MAP_H
#define MAP_H

#include <QGraphicsView>
#include <QGraphicsGeoMap>
#include <QGeoMappingManager>
#include <QGeoServiceProvider>
#include <QGeoCoordinate>
#include <QGeoMappingManagerEngine>
#include <QGeoBoundingBox>
#include <QGeoMapPolylineObject>
#include <QGeoMapCircleObject>
#include <QGraphicsSceneMouseEvent>
#include <QMouseEvent>
#include <QGeoMapPixmapObject>

#include <QDebug>

QTM_USE_NAMESPACE

/**The map class implements a map which displays a route via the gps data.
   It is interactive, the user can zoom and pan it.*/

class map : public QGraphicsGeoMap
{
    Q_OBJECT

    public:
        map(QGeoMappingManager *manager);
        int init(QGraphicsView *gui_view);

        /**Scene to hold the map.*/
        QGraphicsScene *scene;
        /**Graphics view to display the map.*/
        QGraphicsView *view;

        /**Coordinate list, contains the route coordinates.*/
        QList<QGeoCoordinate> coordinate_list;
        /**Polyline to display the route.*/
        QGeoMapPolylineObject *polyline;
        /**Marks the start of the route.*/
        QGeoMapPixmapObject *startmarker;
        /**Marks the end of the route.*/
        QGeoMapPixmapObject *endmarker;
        /**Marks the car on the route.*/
        QGeoMapPixmapObject *carmarker;

        /**Is set to true if the user is panning the map.*/
        bool pan_active;
        /**Is set to true if the car is added to the map.*/
        bool car_added;

        int setRoute(double *longitude, double *latitude, int beginning, int end);
        int setCar(double longitude, double latitude);
        int removeCar();
        int setMarkers(double start_longitude, double start_latitude, double end_longitude, double end_latitude);

        void mousePressEvent(QGraphicsSceneMouseEvent *event);
        void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
        void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
        void wheelEvent(QGraphicsSceneWheelEvent *event);

        bool eventFilter(QObject *obj, QEvent *event);
};

#endif // MAP_H
