/*Copyright (C) 2012 Johannes Hiller, Michael Hoppe, Waldemar Heinrich
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*   you may not use this file except in compliance with the License.
*   You may obtain a copy of the License at
*
*       http://www.apache.org/licenses/LICENSE-2.0
*
*   Unless required by applicable law or agreed to in writing, software
*   distributed under the License is distributed on an "AS IS" BASIS,
*   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*   See the License for the specific language governing permissions and
*   limitations under the License.
*/

package de.tui.itlogger;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Bundle;
import android.text.util.Linkify;
import android.text.util.Linkify.MatchFilter;
import android.text.util.Linkify.TransformFilter;
import android.widget.TextView;

import com.actionbarsherlock.app.ActionBar;
import com.actionbarsherlock.app.SherlockActivity;
import com.actionbarsherlock.view.MenuItem;

/**
 * Class for the about menu
 */
public class MenuAboutActivity extends SherlockActivity {
    
	/**
	 * Called on creation 
	 * @param savedInstanceState load saved data
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.about);
		ActionBar actionbar = getSupportActionBar();
	    actionbar.setDisplayHomeAsUpEnabled(true);
		String title = getString(R.string.abouttitle);
	    actionbar.setTitle(title);
	    displayVersionName();
		/**
		 * transforms the link for the apache license
		 */
		class sampleTransformFilter implements TransformFilter {
		    public String transformUrl(Matcher match1, String url1) {
		        return "http://www.apache.org/licenses/LICENSE-2.0";
		    }
		}
		TextView url =(TextView)findViewById(R.id.url);
		/**
		 * looks for equal strings
		 */
		class sampleMatchFilter implements MatchFilter {
		    public boolean acceptMatch(CharSequence s, int start, int end) {
		        return true;
		    }
		}       
		//Makes apache link a link
		Pattern sampleMatcher = Pattern.compile("http://www.apache.org/licenses/LICENSE-2.0");
		String sampleURL =    "";
		Linkify.addLinks(url, sampleMatcher, sampleURL, new sampleMatchFilter(), new sampleTransformFilter());
		
		/**
		 * Transforms the Link to sourceforge
		 */
		class sampleTransformFilter2 implements TransformFilter {
		    public String transformUrl(Matcher match1, String url1) {
		        return "https://sourceforge.net/projects/project-nine/files/Desktopanwendung/";
		    }
		}
		//Makes "sourceforge" a link
		TextView sourceforge =(TextView)findViewById(R.id.sourceforge);
		Pattern sampleMatcher2 = Pattern.compile("Sourceforge");
		Linkify.addLinks(sourceforge, sampleMatcher2, sampleURL, new sampleMatchFilter(), new sampleTransformFilter2());
		
		/**
		 * Transforms the Link to tu-ilmenau.de
		 */
		class sampleTransformFilter3 implements TransformFilter {
		    public String transformUrl(Matcher match1, String url1) {
		        return "http://www.tu-ilmenau.de/";
		    }
		}
		//Makes "TU Ilmenau" a link
		TextView whatisthis =(TextView)findViewById(R.id.whatisthis);
		Pattern sampleMatcher3 = Pattern.compile("TU Ilmenau");
		Linkify.addLinks(whatisthis, sampleMatcher3, sampleURL, new sampleMatchFilter(), new sampleTransformFilter3());
	}
	/**
	 * Make the icon in the top left corner a up-button
	 * @param MenuItem {@link android.R.id.home}: the up-button 
	 */
	public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
        	case android.R.id.home:
        		// app icon in action bar clicked; go home
        		Intent intent = new Intent(this, iTloggerActivity.class);
        		intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        		startActivity(intent);
        		break;
        }
        return true;
		}
	/**
	 * When called on, it displays the version of the app in a textview
	 */
	private void displayVersionName() {
	    String versionName = "";
	    PackageInfo packageInfo;
	    try {
	        packageInfo = getPackageManager().getPackageInfo(getPackageName(), 0);
	    	versionName = "v " + packageInfo.versionName;
	    } catch (NameNotFoundException e) {
	        e.printStackTrace();
	    }
	    TextView tv = (TextView) findViewById(R.id.version);
	    tv.setText(versionName);
	}
    @Override
    public void onBackPressed() {
    	super.onBackPressed();
    	Intent intent = new Intent(this, iTloggerActivity.class);
		intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		startActivity(intent);
    }
}
