/*Copyright (C) 2012 Johannes Hiller, Michael Hoppe, Waldemar Heinrich
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*   you may not use this file except in compliance with the License.
*   You may obtain a copy of the License at
*
*       http://www.apache.org/licenses/LICENSE-2.0
*
*   Unless required by applicable law or agreed to in writing, software
*   distributed under the License is distributed on an "AS IS" BASIS,
*   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*   See the License for the specific language governing permissions and
*   limitations under the License.
*/

package de.tui.itlogger;

/**
 * @mainpage iTLogger
 * 
 * @section sec1 Introduction
 * This app is part of a software project at the TU Ilmenau in 2012.
 *  Special thanks to our tutor Christoph Bodenstein.
 *  
 *  @section sec1 Startup
 *  On the start of the app, the user sees the {@link SplashActivity}. Here he is asked to
 *  obey the traffic regulations and not use this app while driving.
 *  Some of the variables are also defined here and some others reset.
 *  @section sec2 Runtime
 *  @subsection sec2_1 Main screen
 *  The main screen is the layout of the {@link iTloggerActivity}. Here the user sees the
 *  values of the sensors, GPS and the OBDII connection, if available. The activity also 
 *  inflates the menu and takes care of the camera.
 *  @subsection sec2_2 Preferences
 *  The {@link iTloggerPreferences} infaltes the preferences layout and handles the
 *  callback of the preferences. In the layout the user can change all the settings, e.g.
 *  the settings for databases, sensors, camera and OBDII connections.
 *  {@link explorer} is responsible for deleting data that the user wants to delete
 *  
 *  @subsection sec2_3 About
 *  {@link MenuAboutActivity} shows the about screen.
 *  
 *  @section sec3 Background processes
 *  These are the progresses in the background that handle the refresh of the
 *  sensors, GPS and the database.
 *  @subsection sec3_1 Sensors
 *  {@link Sensoren} handles the calibration of the sensors and the refreshing.
 *  @subsection sec3_2 GPS
 *  {@link GPS} handles the callback of the GPS
 *  @subsection sec3_3 OBDII
 *  {@link OBD2_Activity} and {@link OBD2_Service} handle the initialization of the OBDII
 *  connection and the recieving and formatting of the raw OBDII data. For the connection
 *  {@link DeviceListActivity} is used.
 *  @subsection sec3_4 SQLite
 *  {@link SQLite} handles the writing of the sensor data, GPS data and OBDII data in
 *  the SQLite database*
 */
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.gesture.GestureOverlayView;
import android.hardware.Camera;
import android.hardware.Camera.PictureCallback;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.location.LocationListener;
import android.location.LocationManager;
import android.media.ExifInterface;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.PowerManager;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.Surface;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ViewSwitcher;

import com.actionbarsherlock.app.ActionBar;
import com.actionbarsherlock.app.SherlockActivity;
import com.actionbarsherlock.view.Menu;
import com.actionbarsherlock.view.MenuInflater;
import com.actionbarsherlock.view.MenuItem;

/**
 * Main class of the app, implements the main views, options and camera
 * Has {@link Sensoren} and {@link GPS} as extension
 */
public class iTloggerActivity extends SherlockActivity implements SensorEventListener, android.view.GestureDetector.OnGestureListener {
    /// initialized on the first call upon the activity 
    double longit, latit, spee, alt, course;
    int refresh_sensors_time;
    private SensorManager sensorManager;
    private PowerManager.WakeLock wl;
    public static boolean calibrate = false, calibrated=false, nameset = false, start=false,
    		end = false, exit=false;
	private static final String TAG = "iTLogger";
	private ViewSwitcher switcher;
	private GestureDetector gesturedetector = null;
	int SWIPE_MIN_VELOCITY = 100;
	int SWIPE_MIN_DISTANCE = 100;
	static final String DATEFORMAT = "yyyy-MM-dd HH:mm:ss"; 
	long Date =0;
	public static boolean mScreenOrientationLocked;
	public static final int UPDATE_GUI = 999;
	public boolean update=true;
	private updateGUI mupdateGUI;
	private updateDB mupdateDB;
    
    //Strings for folder declaration
	static String formattedDate = null;
	static String dirName = null; 
	static String path = "/sdcard/";
	static String appname; 
	static String paths = null;
	static String dbnamefolder;
	static String NOMEDIAFILE = ".nomedia";
	static String dcimpath;

	private Camera mCamera;
	private camera mcamera;
    private CameraPreview mPreview;
    private static final int MEDIA_TYPE_IMAGE = 1;
    public boolean takepicture = false;
    public boolean picsaved=true;
    
    static int Rotation;
    static String exifFile;
    static int ORIENTATION_ROTATE_90 =6;
    static int ORIENTATION_ROTATE_NORMAL=1;
    static int stopwatch=0;
    static String title="";
    static File space = new File("/sdcard/");
    public static boolean nospace = false;
    public static boolean firsttime = true;
    static int status;
    static boolean spaceError=false;
    
    /**
     * Initialize the options menu
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getSupportMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        return true;
    }
    /**
     * Define the action on selecting an options item
     * @param item The items of the optionsmenu
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	
        // Handle item selection
        switch (item.getItemId()) {
        	case android.R.id.home:
        		// app icon in action bar clicked; go home
        		Intent intent = new Intent(this, iTloggerActivity.class);
        		intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        		startActivity(intent);
        		break;
        		//The click on the preferences
        	case R.id.preferences:
        		preferences();
        		break;
        		//The click on the calibration button
        	case R.id.calibrate:
        		calibrate();
        		break;
        		//The Click on the about button
        	case R.id.about:
        		about();
        		break;
        		//The exit button
            case R.id.exit:
            	start=false;
            	exit=true;
            	exitall();
            	break;
        }
        return true;
    }
    /**
     * Calls the {@link iTloggerPreferences} if preferences is selected
     * or displays a toast if a recording is in progress, asking the user to 
     * stop the recording first
     * @see iTloggerPreferences
     */
    public void preferences() {
    	if(start==true) {
    		String plsstoprec = getString(R.string.plsstoprec);
			Context context = getApplicationContext();
			int duration = Toast.LENGTH_LONG;
			Toast.makeText(context, plsstoprec, duration).show();
    	} else {
    			Intent intent = new Intent(iTloggerActivity.this, iTloggerPreferences.class);
    			startActivity(intent);
    		}
    }
    /**
     * Calibrates the sensors or displays a toast if a recording is in progress,
     * asking the user to stop the recording first
     */
    public void calibrate () {
    	if(start==true){
    		String plsstoprec = getString(R.string.plsstoprec);
			Context context = getApplicationContext();
			int duration = Toast.LENGTH_LONG;
			Toast.makeText(context, plsstoprec, duration).show();
    	} else{
    		
    		if(nameset==false){
    			String pleasesetname = getString(R.string.pleasesetname);
    			Context context = getApplicationContext();
    			int duration = Toast.LENGTH_LONG;
    			Toast.makeText(context, pleasesetname, duration).show();
    		}else{
    			calibrate = true;
    			calibrated = true;
    			lockScreenOrientation();
    		}
    	}
    }
    /**
     * Shows the about screen when "About" is selected or displays a toast,
     * asking the user to stop the recording first if a recording is in progress
     * @see MenuAboutActivity
     */
    public void about() {
    	if(start==true){
    		String plsstoprec = getString(R.string.plsstoprec);
			Context context = getApplicationContext();
			int duration = Toast.LENGTH_LONG;
			Toast.makeText(context, plsstoprec, duration).show();
    	}else{
    			Intent intent = new Intent(iTloggerActivity.this, MenuAboutActivity.class);
    			startActivity(intent);
    		}
    } 
    /**
     * Called upon, when the app is sent to background, disables the database access
     * and saves the calibrate variables in case the user is only accessing the 
     * preferences
     * Also stops the thread updating the GUI
     * releases the powermanager 
     * @see releaseCamera
     */
    @Override
    protected void onPause() {
    	super.onPause();
    	
    	if(mupdateGUI != null){
			mupdateGUI.cancel();
			mupdateGUI=null;
		}
    	//Release the power manager
    	wl.release();
    	//release the camera
    	releaseCamera(); 
    	
    	Context context = getApplicationContext();
    	SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
    	SharedPreferences.Editor editor = settings.edit();
    	editor.putBoolean("calibrate", calibrate);  
    	editor.putBoolean("calibrated", calibrated);
    	editor.putBoolean("nameset", nameset);
    	editor.commit();
    	start=false;
    }
    /**
     * Resumes the app if it comes to the foreground again
     * Gets the powermanager back
     */
    @Override
    protected void onResume() {
    	super.onResume();
    	wl.acquire();
    	Context context = getApplicationContext();
    	SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
    	calibrate=settings.getBoolean("calibrate", false);  
    	calibrated=settings.getBoolean("calibrated", false);
    	nameset=settings.getBoolean("nameset", false);
    }
    /**
     * Called upon when the app is stopped
     * Saves the state of the calibrate variables and releases the camera
     * @see releaseCamera
     */
    @Override
    protected void onStop() {
    	super.onStop();
    	releaseCamera();
    }

    
    /**
     * Called upon when the activity is started. Defines Layout, tells user to calibrate
     * the sensors, if not yet happened, initializes the PowerManager to keep the display
     * on, initalizes GPS, sensors and checks for the available sensors 
     */
	@Override
    public void onCreate(Bundle savedInstanceState) {
		
		if(exit == true){
			finish();
			System.exit(0);
		}
		
		if(mupdateGUI != null){
			mupdateGUI=null;
		}
		//Start the GUI update thread
		mupdateGUI = new updateGUI();
		mupdateGUI.start();	
		
		final SharedPreferences prefs = PreferenceManager
                .getDefaultSharedPreferences(getApplicationContext());
        iTloggerPreferences.sensorfrequency=prefs.getInt("sensorfreq", 250);
        
        Rotation = getWindowManager().getDefaultDisplay().getRotation();
		
        //Display a toast if the data name isn't set
		if(nameset==false){
			String pleasesetname = getString(R.string.pleasesetname);
			Context context = getApplicationContext();
			int duration = Toast.LENGTH_LONG;
			Toast.makeText(context, pleasesetname, duration).show();
			
		}
		//Display a toast of the name is set but the sensors aren't calibrated
		if(calibrated==false && nameset==true){
			String pleasecalibrate = getString(R.string.pleasecalibrate);
			Context context = getApplicationContext();
			int duration = Toast.LENGTH_LONG;
			Toast.makeText(context, pleasecalibrate, duration).show();
			
		}
		//Keep the screen on
		PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
	    wl = pm.newWakeLock(PowerManager.FULL_WAKE_LOCK, "DoNotDimScreen");
		
	    //set the content view
        setContentView(R.layout.main);
        super.onCreate(savedInstanceState);
        switcher = (ViewSwitcher)findViewById(R.id.viewswitcher);
        gesturedetector = new GestureDetector(this, this);
   
        //Get an instance of the camera
        mCamera = getCameraInstance();

        //Add a camera preview
        mPreview = new CameraPreview(this, mCamera);
        FrameLayout preview = (FrameLayout) findViewById(R.id.camerapreview);
        preview.addView(mPreview);
        
        //Initialize GPS
        LocationManager mlocManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
        LocationListener mlocListener = (LocationListener) new GPS();
        mlocManager.requestLocationUpdates( LocationManager.GPS_PROVIDER, 1000, 0, mlocListener);
        
        //Initialize sensors
        sensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
        
        //Check if sensors are available
        if (sensorManager.getSensorList(Sensor.TYPE_ACCELEROMETER).size() != 0){
        sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_NORMAL);
        }
        if (sensorManager.getSensorList(Sensor.TYPE_GYROSCOPE).size() != 0){
		sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_GYROSCOPE),
				SensorManager.SENSOR_DELAY_NORMAL);
        }
        //Initialize the start and stop button
        final Button startstopbutton = (Button)findViewById(R.id.startstop);
        startstopbutton.setTag(1);
        startstopbutton.setText(R.string.start);
        startstopbutton.setOnClickListener(new OnClickListener() {
        	public void onClick(View v) {
        		if (spaceError==true) {
        			status =1;
        			if (firsttime==true) {
        				spaceError=false;
        			}
        		}
        		else {
        		status =(Integer) v.getTag();
        		}
            	if(status == 1) {	
            	if(firsttime==true){	
            		if (nameset==true) {
            			if(calibrated==true && nameset==true){
            				//If OBDII is set to active in the Preferences
            				//you can't start a recording until the bluetooth
            				//device is connected and initialized
            				if (iTloggerPreferences.obdactive==true) {
            					if (iTloggerPreferences.ready == false) {
            						String plswaitforobd = getString(R.string.plswaitobd);
            						Context context = getApplicationContext();
            						int duration = Toast.LENGTH_LONG;
            						Toast.makeText(context, plswaitforobd, duration).show();
            					}else {
            						start=true;
            						takepicture=true;
            						lockScreenOrientation();
            						if (iTloggerPreferences.camactive==true) {
            							mcamera = new camera();
            							mcamera.start();
            						}
            					}
            				} else {
            					start=true;
            					takepicture=true;
            					lockScreenOrientation();
        						if (iTloggerPreferences.camactive==true) {
        							mcamera = new camera();
        							mcamera.start();
        						}
            				}
            				//Create a folder for the database access
            				//and creates a .nomedia file
            				if(start == true) {
            					appname=getString(R.string.app_name);
            					(new File(path + appname)).mkdir();
            					paths =path + appname+"/";
            					Calendar c = Calendar.getInstance(); 
            					SimpleDateFormat dsf = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss-SSS"); 
            					formattedDate = dsf.format(c.getTime()); 
            					dirName = dbnamefolder+"_"+formattedDate; 
            					(new File(paths + dirName)).mkdir(); 
            					//Create a .nomedia file in the DCIM folder for
            					//pictures not to appear in the gallery of 
            					//the device
            					dcimpath=paths+dirName+"/"+"DCIM";
            					(new File(dcimpath)).mkdir();
            					File file = new File(dcimpath,NOMEDIAFILE);
            					if (!file.exists()){
            						try {
            							file.createNewFile();
            						} catch (IOException e) {
            							e.printStackTrace();
            						}
            					}
            				
            					SQLite.onCreateDBAndDBTabled();
                				
                				//starts the database update thread
                				mupdateDB = new updateDB();
                				mupdateDB.start();
                				
                				
            				}
            				startstopbutton.setText(R.string.stop);
            				v.setTag(0); //pause
            				
            			}else {
            				//Tells the user to calibrate the sensors
            				String calibrate = getString(R.string.pleasecalibrate);
            				Context context = getApplicationContext();
            				int duration = Toast.LENGTH_LONG;
            				Toast.makeText(context, calibrate, duration).show();
            				Log.d(TAG, "nicht gesetzt: "+calibrated+" "+nameset);
            			}
            			} else {
            				//Tells the user to set a name for the folder
            				String pleasesetname = getString(R.string.pleasesetname);
            				Context context = getApplicationContext();
            				int duration = Toast.LENGTH_LONG;
            				Toast.makeText(context, pleasesetname, duration).show();
            				
            			}
            	}else{
            		String plsdeletefolders = getString(R.string.plsdeletefolders);
    				Context context = getApplicationContext();
    				int duration = Toast.LENGTH_LONG;
    				Toast.makeText(context, plsdeletefolders, duration).show();
            	}
            		} else {
            			//Displays the dialog in case the recording is in progress
            			AlertDialog diabox = areyousure();
            			diabox.show();
            			unlockScreenOrientation();
            			v.setTag(1); //pause
            		}
            	
        		}
        	});
	}
	/**
	 * Alert dialog shown when users wants to stop recording, has
	 * options yes and cancel, makes sure that recording is not accadentily 
	 * stopped
	 */
	private AlertDialog areyousure() {
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		String ays = getString(R.string.areyousuretext);
		String yes = getString(R.string.yes);
		String cancel = getString(R.string.cancel);
		builder.setMessage(ays)
				//User can only choose between cancel and yes
		       .setCancelable(false)
		       //Stops the recording in case of yes
		       .setPositiveButton(yes, new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   Button startstopbutton = (Button)findViewById(R.id.startstop);
               			startstopbutton.setText(R.string.start);
            			start=false;
            			stopwatch=0;
            			takepicture=false;
            			unlockScreenOrientation();
            			calibrate=false;
            			calibrated=false;
            			if (mcamera != null) mcamera=null;
            			if (mupdateDB != null) mupdateDB=null;
               			return;
		           }
		       })
		       //cancels the dialog
		       .setNegativeButton(cancel, new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                dialog.cancel();
		           }
		       });
		AlertDialog alert = builder.create();
		return alert;
	}
	
	private AlertDialog fullspace() {
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		String ays = getString(R.string.sdfull);
		String yes = getString(R.string.yes);
		String cancel = getString(R.string.cancel);
		builder.setMessage(ays)
				//User can only choose between cancel and yes
		       .setCancelable(false)
		       //Stops the recording in case of yes
		       .setPositiveButton(yes, new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   Intent intent = new Intent(iTloggerActivity.this, explorer.class);
			    		startActivity(intent);
			    		status=1;
			    		spaceError=true;
               			return;
		           }
		       })
		       //cancels the dialog
		       .setNegativeButton(cancel, new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                dialog.cancel();
		                status=1;
		                spaceError=true;
		           }
		       });
		
		AlertDialog alert = builder.create();
		return alert;
	}
	/** 
	 * Calls the gesturedetector
	 * @param event A motionevent, in this cases flinging sideways on the screen
	 * @return an ontouchevent to the gesturedetector
	 */
	@Override
	public boolean onTouchEvent(MotionEvent event){
		return gesturedetector.onTouchEvent(event);
	}
	/**
	 * Handler for handling the output on the screen
	 */
	private final Handler mdataHandler = new Handler() {
	      
		/**
		 * Updates the GUI and outputs the data on the screen
		 */
		@Override
        public void handleMessage(Message msg) {
        	
            switch (msg.what) {
            case UPDATE_GUI:{
	        	//output
            	
	        	if(start==true)
	        	{	
	        		long freeSpace = space.getFreeSpace();
	            	float totalSpace = space.getTotalSpace(); //total disk space in bytes.
	            	float usableSpace = space.getUsableSpace(); ///unallocated / free disk space in Mbyte.
	            	float procent = (usableSpace/totalSpace)*100;
	            	int prozent = (int) procent;
	            	
	            	if(firsttime==true){
	            	if(freeSpace < 104857600){
	            		AlertDialog diabox = fullspace();
            			diabox.show();
            			 firsttime=false;
            			 spaceError=true;
            			 Button startstopbutton = (Button)findViewById(R.id.startstop);
                			startstopbutton.setText(R.string.start);
                			start=false;
                			stopwatch=0;
                			takepicture=false;
                			unlockScreenOrientation();
                			status=1;
                			calibrate=false;
                			calibrated=false;
             			if (mcamera != null) mcamera=null;
             			if (mupdateDB != null) mupdateDB=null;
            			 
	            		}
	            	}
	            	
	        	stopwatch++;
	        	title=String.valueOf(stopwatch/4);
            	
	        	//Initializes the action bar and displays the stopwatch in the title
            	final ActionBar actionBar = getSupportActionBar();
		        actionBar.setTitle(getString(R.string.recsince)+" "+ title +"s");
		        actionBar.setSubtitle(prozent+getString(R.string.freespace));
	        	}
	        	
	        	//Gyroscope output
	    		TextView gyrx=(TextView)findViewById(R.id.gyrx);
	    		String xaxis = getString(R.string.xaxis);
	    		gyrx.setText(xaxis+": " + DivDecimal(Sensoren.values_g[0],2));
	            TextView gyry=(TextView)findViewById(R.id.gyry);
	            String yaxis = getString(R.string.yaxis);
	            gyry.setText(yaxis+": " + DivDecimal(Sensoren.values_g[1],2));	
	            TextView gyrz=(TextView)findViewById(R.id.gyrz);
	            String zaxis = getString(R.string.zaxis);
	            gyrz.setText(zaxis+": " + DivDecimal(Sensoren.values_g[2],2));
	    		//Sensor output
	    		TextView accx=(TextView)findViewById(R.id.accx);
	            accx.setText(xaxis+": " + DivDecimal(Sensoren.values_a[0],2));	
	            TextView accy=(TextView)findViewById(R.id.accy);
	            accy.setText(yaxis+": " + DivDecimal(Sensoren.values_a[1],2));	
	            TextView accz=(TextView)findViewById(R.id.accz);
	            accz.setText(zaxis+": " + DivDecimal(Sensoren.values_a[2],2));		
	            //GPS output
	            TextView longi=(TextView)findViewById(R.id.longitude);
	            String longitude = getString(R.string.longitude);
	            longi.setText(longitude+": "+ DivDecimal(GPS.gps_data[0],6));
	            TextView lat=(TextView)findViewById(R.id.lat);
	            String latitude = getString(R.string.latitude);
	            lat.setText(latitude+": "+ DivDecimal(GPS.gps_data[1],6));
	            TextView velo=(TextView)findViewById(R.id.speed);
	            String speed = getString(R.string.speed);
	            velo.setText(speed+": "+ DivDecimal(GPS.gps_data[2],2)+" m/s"); 
	            TextView alti=(TextView)findViewById(R.id.altitude);
	            String altitude = getString(R.string.altitude);
	            alti.setText(altitude+": "+ GPS.gps_data[3]+" m"); 
	            TextView cour=(TextView)findViewById(R.id.cour);
	            String cours = getString(R.string.course);
	            cour.setText(cours+": "+ DivDecimal(GPS.gps_data[4],2)+" \u00b0");
	            //OBDII output
	            //Engine RPM
	            String rpmv = getString(R.string.enginerpm);
	            TextView RPM=(TextView)findViewById(R.id.RPM);
	            if((OBD2_Activity.selected[0] == true) && (OBD2_Activity.OBD_values[0] != -255)){
	            	RPM.setText(rpmv+": " + OBD2_Activity.OBD_values[0]+" RPM");
	            }else{
	            	RPM.setText(rpmv+": " + getString(R.string.notselected));	
	            }
	            //Vehicle Speed
	            String vs = getString(R.string.vehiclespeed);
	            TextView Speed=(TextView)findViewById(R.id.Speed);
	            if((OBD2_Activity.selected[1] == true) && (OBD2_Activity.OBD_values[1] != -255)){
	            	Speed.setText(vs+": " + OBD2_Activity.OBD_values[1]+" km/h");
	            }else{
	            	Speed.setText(vs+": " + getString(R.string.notselected));	
	            }   
	            //Throttle position
	            String tp = getString(R.string.throttlepos);
	            TextView throttle=(TextView)findViewById(R.id.Throttle);
	            if((OBD2_Activity.selected[2] == true) && (OBD2_Activity.OBD_values[2] != -255)){
	            	throttle.setText(tp+": " + OBD2_Activity.OBD_values[2]+" %");
	            }else{
	            	throttle.setText(tp+": " + getString(R.string.notselected));	
	            }
	            //Barometric pressure
	            String bp = getString(R.string.barpressure);
	            TextView BarometricPressure=(TextView)findViewById(R.id.barometricpressure);
	            if((OBD2_Activity.selected[3] == true) && (OBD2_Activity.OBD_values[3] != -255)){
	            	BarometricPressure.setText(bp+": " + OBD2_Activity.OBD_values[3]+" kPa");
	            }else{
	            	BarometricPressure.setText(bp+": " + getString(R.string.notselected));	
	            } 
	            //Ambient air temperature
	            String at = getString(R.string.airtemp);
	            TextView Ambienttemp=(TextView)findViewById(R.id.Ambienttemp);
	            if((OBD2_Activity.selected[4] == true) && (OBD2_Activity.OBD_values[4] != -255)){
	            	Ambienttemp.setText(at+": " + OBD2_Activity.OBD_values[4]+" \u00b0"+"C");
	            }else{
	            	Ambienttemp.setText(at+": " + getString(R.string.notselected));	
	            } 
	            //Percentage of the engine reference torque 
	            String ept = getString(R.string.torque);
	            TextView EnginePercentTorque=(TextView)findViewById(R.id.percenttorque);
	            if((OBD2_Activity.selected[5] == true) && (OBD2_Activity.OBD_values[5] != -255)){
	            	EnginePercentTorque.setText(ept+": " + OBD2_Activity.OBD_values[5]+" %");
	            }else{
	            	EnginePercentTorque.setText(ept+": " + getString(R.string.notselected));		
	            }
	            //Engine reference torque
	            String ert = getString(R.string.reftorque);
	            TextView ReferenceTorque=(TextView)findViewById(R.id.referencetorque);
	            if((OBD2_Activity.selected[6] == true) && (OBD2_Activity.OBD_values[6] != -255)){
	            	ReferenceTorque.setText(ert+": " + OBD2_Activity.OBD_values[6]+" Nm");
	            }else{
	            	ReferenceTorque.setText(ert+": " + getString(R.string.notselected));		
	            }
	            //Intake air temperature
	            String iat = getString(R.string.intakeairtemp);
	            TextView intakeairtemp=(TextView)findViewById(R.id.Intakeairtemp);
	            if((OBD2_Activity.selected[7] == true) && (OBD2_Activity.OBD_values[7] != -255)){
	            	intakeairtemp.setText(iat+": " + OBD2_Activity.OBD_values[7]+" \u00b0"+"C");
	            }else{
	            	intakeairtemp.setText(iat+": " + getString(R.string.notselected));	
	            }
	            //Vehicle ID
	            String vehid = getString(R.string.vehid);
	            TextView dispvehid = (TextView)findViewById(R.id.vehicleid);
	            if(OBD2_Activity.selected[0] == true){
	            	dispvehid.setText(vehid+": " +OBD2_Activity.VID);
	            }else{
	            	dispvehid.setText(vehid+": " + getString(R.string.notselected));		
	            }
            }
	            break;
            }
        }
    }; 
    
    
    /**
     * Refreshes the GUI every 250ms
     */
    private class updateGUI extends Thread{
    	public boolean update_thread=true;
    	public void cancel(){
    		update_thread=false;
    	}
    	/**
    	 * The thread for updating the GUI
    	 */
    	public void run() {
    	
    		while(update_thread == true){
    		mdataHandler.obtainMessage(UPDATE_GUI).sendToTarget();
    		
    		try {
	  			Thread.sleep(250);
	  		} catch (InterruptedException e) {
	  			e.printStackTrace();
	  		}
    		}
    		
    	}
    }
    /**
     * Calls the the {@link writeDB} every few seconds, depending
     * on what the user set in the preferences
     * at the end of the runtime of the app, the database is closed here
     */
    private class updateDB extends Thread{
    	public void run() {
    		SQLite.db = SQLiteDatabase.openOrCreateDatabase(paths+iTloggerActivity.dbnamefolder+"_"+iTloggerActivity.formattedDate+"/"+"data.sqlite", null); 
			Cursor c1 = SQLite.db.rawQuery("PRAGMA journal_mode = MEMORY", null); c1.close();
    		
			//Only writes to the database if start is true
    		while(start == true){
    			if(SQLite.ready==true){
    			SQLite.writeDB();
    		try {
	  			Thread.sleep(iTloggerPreferences.sensorfrequency);
	  		} catch (InterruptedException e) {
	  			e.printStackTrace();
	  		}
    		} else {
    			try {
    	  			Thread.sleep(5);
    	  		} catch (InterruptedException e) {
    	  			e.printStackTrace();
    	  		}
    		}
    		}
    		//closes the database after start is false
    		SQLite.db.close();
    	}
    }

	public void onAccuracyChanged(Sensor sensor, int accuracy) {
	}
	protected void update(String tableMid, String string, Object object) {
		// TODO Auto-generated method stub
	}
	/**
	 * Takes care of the sensors if they are calibrated and change
	 */
	public void onSensorChanged(SensorEvent event) {
		if(calibrated == true){
			//Checks which sensor changes
			if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
				Sensoren.getAccelerometer(event);
			}
			if (event.sensor.getType() == Sensor.TYPE_GYROSCOPE) {
				Sensoren.getGyroscope(event);
			}
		}
	}
	/**
	 * Divides the GPS and sensor values for them to be correctly displayed on
	 * the screen
	 */
	public float DivDecimal(int help, int decimalPlace) {
        float value=0;
        value = (float) (help/(Math.pow(10, decimalPlace)));
        return value;
	}
	/**
	 * Called upon when the app is about to be destroyed
	 */
	public void onDestroy() {
		super.onDestroy();
		releaseCamera();
	}
	public void onGesture(GestureOverlayView arg0, MotionEvent arg1) {
		// TODO Auto-generated method stub
	}
	public void onGestureCancelled(GestureOverlayView arg0, MotionEvent arg1) {
		// TODO Auto-generated method stub
	}
	public void onGestureEnded(GestureOverlayView arg0, MotionEvent arg1) {
		// TODO Auto-generated method stub
	}
	public void onGestureStarted(GestureOverlayView arg0, MotionEvent arg1) {
		// TODO Auto-generated method stub
	}
	public boolean onDown(MotionEvent arg0) {
		// TODO Auto-generated method stub
		return false;
	}
	/**
	 * Method for flinging between the two views
	 */
	public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX,
			float velocityY	) {
		float ev1X = e1.getX();
		float ev2X = e2.getX();
		final float xdistance = Math.abs(ev1X - ev2X);
        final float xvelocity = Math.abs(velocityX);
        if( (xvelocity > SWIPE_MIN_VELOCITY) && (xdistance > SWIPE_MIN_DISTANCE) )
        {
			if(ev1X > ev2X)//Switch Left
			{
				previousView();
			} else//Switch Right
			{
				nextView();
			}
        }
		return false;
	}
	public void onLongPress(MotionEvent arg0) {
		// TODO Auto-generated method stub
	}
	public boolean onScroll(MotionEvent arg0, MotionEvent arg1, float arg2,
			float arg3) {
		// TODO Auto-generated method stub
		return false;
	}
	public void onShowPress(MotionEvent arg0) {
		// TODO Auto-generated method stub
	}
	public boolean onSingleTapUp(MotionEvent arg0) {
		// TODO Auto-generated method stub
		return false;
	}
		/**
		 * Implements previousView from viewswitcher and gives it an animation
		 */
		public void previousView() {
			
				//Previous View
				switcher.setInAnimation(this, R.anim.in_animation1);
				switcher.setOutAnimation(this, R.anim.out_animation1);
				switcher.showPrevious();
		}
		/**
		 * Implements nextView from ViewSwitcher and gives it an animation
		 */
		public void nextView() {
			
				//Next View
				switcher.setInAnimation(this, R.anim.in_animation);
				switcher.setOutAnimation(this, R.anim.out_animation);
				switcher.showNext();
		}
		/**
		 * Locks the screen orientation on pressing the calibrating button to the
		 * current screen orientation
		 */
		public void lockScreenOrientation() {
		    if (!mScreenOrientationLocked) {
		        final int orientation = getResources().getConfiguration().orientation;
		        final int rotation = getWindowManager().getDefaultDisplay().getOrientation();
		        
		        if (rotation == Surface.ROTATION_0 || rotation == Surface.ROTATION_90) {
		            if (orientation == Configuration.ORIENTATION_PORTRAIT) {
		                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		            } else if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
		                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		            }
		            
		        } else if (rotation == Surface.ROTATION_180 || rotation == Surface.ROTATION_270) {
		            if (orientation == Configuration.ORIENTATION_PORTRAIT) {
		                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT);
		            } else if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
		                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE);
		            }
		        }
		        mScreenOrientationLocked = true;
		    }
		}
	    /**
	     * Unlocks the screen orientation
	     */
		public void unlockScreenOrientation() {
		    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR);
		    mScreenOrientationLocked = false;
		}
		/**
		 *  opens an Instance of the Camera
		 */
		public static Camera getCameraInstance(){
		    Camera c = null;
		    try {
		        c = Camera.open(); // attempt to get a Camera instance
		    }
		    catch (Exception e){
		    	c.release();
		    }
		    return c; 
		}
		/** 
		 * The camera preview 
		 */
		public class CameraPreview extends SurfaceView implements SurfaceHolder.Callback {
		    private SurfaceHolder mHolder;
		    private Camera mCamera;
		    /**
		     * Camera preview
		     * Gets the holder 
		     * @param context application context
		     * @param camera the camera that is used
		     */
		    @SuppressWarnings("deprecation")
		    public CameraPreview(Context context, Camera camera) {
		        super(context);
		        mCamera = camera;
		        mHolder = getHolder();
		        mHolder.addCallback(this);
		        mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
		    }
		    /**
		     *  Shows the preview of the Camera and changes the settings
		     * @param holder 
		     */
		    public void surfaceCreated(SurfaceHolder holder) {
		    	if (mCamera!=null) {
		        try {
		            Camera.Parameters parameters = mCamera.getParameters();
			        parameters.setPictureSize(iTloggerPreferences.w, iTloggerPreferences.h);
			        parameters.set("jpeg-quality", 75);
			        mCamera.setParameters(parameters);
			        //Portrait
			        if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT) {
			        	mCamera.setDisplayOrientation(90);
	                }
			        //Landscape
	                if (getResources().getConfiguration().orientation ==Configuration.ORIENTATION_LANDSCAPE) {
			        	mCamera.setDisplayOrientation(0);
	                }
			        mCamera.setPreviewDisplay(holder);
		            mCamera.startPreview();
		        } catch (IOException e) {
		        	mCamera.release();
		           
		        }
		    	} else {
		    		mCamera.release();
		    	}
		    }
		    /** 
		     * Handles the end of the surface
		     */
		    public void surfaceDestroyed(SurfaceHolder holder) {
		    }
		    /** 
		     * Handles changes in the Setting 
		     * @param format The format of the picture
		     * @param w The width of the pictures
		     * @param h The height of the pictures
		     */
		    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {

		        if (mHolder.getSurface() == null){
		          return;
		        } 
		        try {
		            mCamera.stopPreview();
		        } catch (Exception e){
		        }
		        try {
		            mCamera.setPreviewDisplay(mHolder);
		            mCamera.startPreview();

		        } catch (Exception e){
		           
		        }
		    }
		}
		/**
		 * Takes the pictures and saves them and writes the GPS data in the EXIF
		 */
		public PictureCallback mPicture = new PictureCallback() {
		    public void onPictureTaken(byte[] data, Camera camera) {
		        File pictureFile = getOutputMediaFile(MEDIA_TYPE_IMAGE);
		        if (pictureFile == null){
		            Log.d(TAG, "Error creating media file, check storage permissions: ");
		            return; 
		        }
		        try {
		            FileOutputStream fos = new FileOutputStream(pictureFile);
		            fos.write(data);
		            fos.close();
		            Log.d(TAG, "Picture saved");
		            picsaved=true;
		        } catch (FileNotFoundException e) {
		            Log.d(TAG, "File not found: " + e.getMessage());
		        } catch (IOException e) {
		            Log.d(TAG, "Error accessing file: " + e.getMessage());
		        }
		        //Writes GPS to the EXIF files
		        ExifInterface exif;
			    try {
			    	// Write GPS data in DMS format to exifs of pictures
			    	double lat = DivDecimal(GPS.gps_data[1],6);
			    	double longi = DivDecimal(GPS.gps_data[0],6);
			        exif = new ExifInterface(exifFile);
			        //Transforms the latitude from decimal to degrees, minutes, seconds
			        int num1Lat = (int)Math.floor(lat);
			        int num2Lat = (int)Math.floor((lat - num1Lat) * 60);
			        double num3Lat = (lat - ((double)num1Lat+((double)num2Lat/60))) * 3600000;
			        //Transforms the longitude from decimal to degrees, minutes, seconds
			        int num1Lon = (int)Math.floor(longi);
			        int num2Lon = (int)Math.floor((longi - num1Lon) * 60);
			        double num3Lon = (longi - ((double)num1Lon+((double)num2Lon/60))) * 3600000;
			        
			        exif.setAttribute(ExifInterface.TAG_GPS_LATITUDE, num1Lat+"/1,"+num2Lat+"/1,"+num3Lat+"/1000");
			        exif.setAttribute(ExifInterface.TAG_GPS_LONGITUDE, num1Lon+"/1,"+num2Lon+"/1,"+num3Lon+"/1000");
			        
			        //Writes N or S according to the location
			        if (GPS.gps_data[1] > 0) {
			            exif.setAttribute(ExifInterface.TAG_GPS_LATITUDE_REF, "N"); 
			        } else {
			            exif.setAttribute(ExifInterface.TAG_GPS_LATITUDE_REF, "S");
			        }
			        //Writes E or W according to location
			        if (GPS.gps_data[0] > 0) {
			            exif.setAttribute(ExifInterface.TAG_GPS_LONGITUDE_REF, "E");    
			        } else {
			        exif.setAttribute(ExifInterface.TAG_GPS_LONGITUDE_REF, "W");
			        }
			        exif.setAttribute(ExifInterface.TAG_MAKE, "iTLogger");
			        exif.saveAttributes();

			    } catch (IOException e) {
			        Log.e("PictureActivity", e.getLocalizedMessage());
			    }
		    }
		};
		/** 
		 * Creates a uri for saving the files
		 * @param type the type of media, in this case {@link MEDIA_TYPE_IMAGE}
		 */
		public static Uri getOutputMediaFileUri(int type){
		      return Uri.fromFile(getOutputMediaFile(type));
		}
		/** 
		 * Defines where and with which name the picture is saved
		 * @param type defines the media type, in this case only {@link MEDIA_TYPE_IMAGE}
		 */
		public static File getOutputMediaFile(int type){
		    File mediaStorageDir = new File(paths+dirName+File.separator+"DCIM");
		    if (! mediaStorageDir.exists()){
		        if (! mediaStorageDir.mkdirs()){
		            Log.d("iTLoggerCamera", "failed to create directory");
		            return null;
		        }
		    }
		    String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
		    File mediaFile;
		    if (type == MEDIA_TYPE_IMAGE){
		        mediaFile = new File(mediaStorageDir.getPath() + File.separator +
		        		"iTL_"+ timeStamp + ".jpg");
		        Log.d(TAG, "mediafile: "+mediaFile);
			    exifFile = mediaFile.toString();
		    } else {
		        return null;
		    }
		    return mediaFile;
		}
		/**
		 * Releases the camera, so that other apps can use the camera
		 */
		public void releaseCamera(){
	        if (mCamera != null){
	            mCamera.release();        
	            mCamera = null;
	        }
	    }
		/**
		 * Thread for regulating the picturefrequency
		 * Restarts the camera preview before taking a picture
		 * in order for it to work on Samsung devices
		 */
		public class camera extends Thread{
			public void run() {
				while (takepicture==true && mCamera!= null){
					if(picsaved == true){
					picsaved=false;
					mCamera.startPreview();
					try {
						Thread.sleep(50);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					mCamera.takePicture(null, null, mPicture);
					try {
						Thread.sleep(((iTloggerPreferences.picturefrequency+1)*1000)-50);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					}else{
						try {
							Thread.sleep(50);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
				mCamera.stopPreview();
				mCamera.startPreview();
			}
		};
		/**
		 *Asks the user if he really wants to quit if he presses the back button 
		 */
		public void onBackPressed() {
	    		    	
	    		new AlertDialog.Builder(this)
	    		.setIcon(R.drawable.ic_launcher)
	    		.setTitle(R.string.exittitle)
	    		.setNegativeButton(R.string.cancel, 
	    				new DialogInterface.OnClickListener(){
				
	    			public void onClick(DialogInterface dialog, int which) 
	    			{
	    				// TODO Auto-generated method stub
	    				dialog.cancel();
	    			}
	    		})
	    		.setPositiveButton(R.string.yes, 
	    				new DialogInterface.OnClickListener(){
					
					public void onClick(DialogInterface dialog, int which) 
					{
						// TODO Auto-generated method stub
						start=false;
						exit=true;
			            exitall();	   
					}
	    		}).show();
	    }
		
		public void exitall(){
			Intent intent = new Intent(this, OBD2_Activity.class);
    		intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
    		startActivity(intent);
		}
}