/*Copyright (C) 2012 Johannes Hiller, Michael Hoppe, Waldemar Heinrich
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*   you may not use this file except in compliance with the License.
*   You may obtain a copy of the License at
*
*       http://www.apache.org/licenses/LICENSE-2.0
*
*   Unless required by applicable law or agreed to in writing, software
*   distributed under the License is distributed on an "AS IS" BASIS,
*   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*   See the License for the specific language governing permissions and
*   limitations under the License.
*/

package de.tui.itlogger;


import java.util.List;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.hardware.Camera;
import android.os.Build;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.preference.PreferenceManager;
import android.preference.SwitchPreference;
import android.util.Log;

import com.actionbarsherlock.app.ActionBar;
import com.actionbarsherlock.app.SherlockPreferenceActivity;
import com.actionbarsherlock.view.MenuItem;

/**
 * Class responsible for the preferences menu
 * @see SherlockPreferenceActiviy
 */
@TargetApi(14)
public class iTloggerPreferences extends SherlockPreferenceActivity {
    protected static final int sensor = 0;
    boolean obd;
	static String namedb;
	static int sensorfrequency;
	static boolean nameset = false;
	static int picturefrequency;
    static CharSequence resolutions[] = new CharSequence[4];
    public static List <Camera.Size> size = null;
    public static int lowheight, lowwidth, maxheight, maxwidth, 
    					middlewidth, middleheight, highheight, highwidth;
    public static int w, h;
    static boolean obdactive;
    static boolean camactive;
	static boolean ready;
	static String[] files;

	/**
	 * Gets the layout from the preferences.xml and initalizes all the 
	 * {@link OnPreferenceClickListener}s and handles the callback
	 */
    @SuppressLint("NewApi")
	@SuppressWarnings("deprecation")
	@Override
	protected void onCreate (Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
        addPreferencesFromResource(R.layout.preferences);
        
        //Initialize the action bar and set the title to "Preferences"
        String title = getString(R.string.preferences);
        final ActionBar actionBar = getSupportActionBar();
        actionBar.setDisplayHomeAsUpEnabled(true);
        actionBar.setTitle(title);
        
        w=lowwidth;
        h=lowheight;
        
        final SharedPreferences prefs = PreferenceManager
                .getDefaultSharedPreferences(getApplicationContext());
        sensorfrequency=prefs.getInt("sensorfreq", 250);
        
        /**
         * Get the string out of the edittext and set nameset to true 
         */
        final EditTextPreference editfolder= (EditTextPreference)findPreference("savefolder");
        editfolder.setOnPreferenceChangeListener(new OnPreferenceChangeListener(){
        	
        	/**
        	 *Called if the value of the preference has changed 
        	 */
			public boolean onPreferenceChange(Preference preference,
					Object newValue) {
				iTloggerActivity.dbnamefolder=(String) newValue;
				iTloggerActivity.nameset=true;
		    	Context context = getApplicationContext();
				SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
		    	SharedPreferences.Editor editor = settings.edit();
		    	editor.putBoolean("nameset", iTloggerActivity.nameset);
		    	editor.commit();
				return false;
			}
        	
        });
		
        /**
         * Displays a list of available frequencies 
         */
        final ListPreference sensorfreqpref = (ListPreference)findPreference("sensorfreqpref");
        sensorfreqpref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {

			public boolean onPreferenceChange(Preference preference,
					Object newValue) {
				String valfreq = (String) newValue;
				int v= sensorfreqpref.findIndexOfValue(valfreq);
				sensorfreqpref.setValueIndex(v);
				//Multiply by 50 and add 50, gives a range from 50ms to 1000ms
				sensorfrequency=(v*50)+50;
			    SharedPreferences.Editor editor = prefs.edit();
			    editor.putInt("sensorfreq", sensorfrequency);
			    editor.commit();
				return false;
			}
        });
        
        /**
         *Displays a list of three picture frequencies 
         */
        final ListPreference campreffreq = (ListPreference)findPreference("camfreqpref");
        campreffreq.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {

			public boolean onPreferenceChange(Preference preference,
					Object newValue) {
				String valfreq = (String) newValue;
				int v= campreffreq.findIndexOfValue(valfreq);
				campreffreq.setValueIndex(v);
				//Add 1, gives 1s, 2s and 3s
				picturefrequency=v+1;
				return false;
			}
        });
        
       /**
        *Gives an intent to open the explorer for deleting recordings 
        *{@see explorer}
        */
        Preference deletefolder = (Preference)findPreference("deletefolder");
        deletefolder.setOnPreferenceClickListener(new OnPreferenceClickListener() {

			public boolean onPreferenceClick(Preference preference) {	
		        	Intent intent = new Intent(iTloggerPreferences.this, explorer.class);
		    		startActivity(intent);
				return true;
			}
        });
        
        /**
         * Calls on different layout depending of the android version since 
         * pre-ICS does not support the nice ViewSwitcher
         */
        if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.HONEYCOMB_MR2) {
        	//Show a Checkbox on devices <API14
        	final CheckBoxPreference camtoggle  = (CheckBoxPreference)findPreference("camactivepref");
        	camtoggle.setOnPreferenceClickListener(new OnPreferenceClickListener() {
        		
        		/**
        		 *Set camactive to true if toggled
        		 *if true, recording can't be started until OBDII is done
        		 *initializing  
        		 */
				public boolean onPreferenceClick(Preference preference) {
					camactive  = camtoggle.isChecked();
					return false;
				}
        	});
        	camactive  = camtoggle.isChecked();
			Log.i("Camactive", ""+camactive);
        } else {
        	//Show a switch on Devices >=API14
            final SwitchPreference camtoggleswitch = (SwitchPreference)findPreference("camactivepref");
            camtoggleswitch.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            	
        		/**
        		 *Set camactive to true if toggled
        		 *if true, recording can't be started until OBDII is done
        		 *initializing  
        		 */
				public boolean onPreferenceClick(Preference preference) {
					camactive  = camtoggleswitch.isChecked();
					return false;
				}
        	});
			camactive  = camtoggleswitch.isChecked();
			Log.i("Camactive", ""+camactive);
        }
        
        /**
         *Displays a list of 4 resolutions according to the resolutions of
         *the device camera
         *Resolutions are polled in {@link SplashActivity} 
         */
        final ListPreference camrespref = (ListPreference)findPreference("camresolutionpref");
        camrespref.setEntries(resolutions);
        camrespref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {

			public boolean onPreferenceChange(Preference preference,
					Object newValue) {
				String valres = (String) newValue;
		    	int v = camrespref.findIndexOfValue(valres);
		    	if (v==0) {
		    		h=lowheight;
		    		w=lowwidth;
		    		camrespref.setValueIndex(v);
		    	}
		    	if (v==1) {
		    		h=middleheight;
		    		w=middlewidth;
		    		camrespref.setValueIndex(v);
		    	}
		    	if (v==2) {
		    		h=highheight;
		    		w=highwidth;
		    		camrespref.setValueIndex(v);
		    	}		
		    	if (v==3) {
		    		h=maxheight;
		    		w=maxwidth;
		    		camrespref.setValueIndex(v);
		    	}		
		    	return false;
			}
        });
        
        /**
         * Calls on different layout depending of the android version since 
         * pre-ICS does not support the nice ViewSwitcher
         */
        if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.HONEYCOMB_MR2) {
        	//Show a Checkbox on devices <API14
        	final CheckBoxPreference obdtoggle = (CheckBoxPreference)findPreference("toggleobd2");
        	obdtoggle.setOnPreferenceClickListener(new OnPreferenceClickListener() {
        		
        		/**
        		 *Set obdactive to true if toggled
        		 *if true, recording can't be started until OBDII is done
        		 *initializing  
        		 */
				public boolean onPreferenceClick(Preference preference) {
					obdactive  = obdtoggle.isChecked();
					ready=false;
					return false;
				}
        	});
        } else {
        	//Show a switch on Devices >=API14
            final SwitchPreference obdtoggleswitch = (SwitchPreference)findPreference("toggleobd2");
            obdtoggleswitch.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            	
        		/**
        		 *Set obdactive to true if toggled
        		 *if true, recording can't be started until OBDII is done
        		 *initializing  
        		 */
				public boolean onPreferenceClick(Preference preference) {
					obdactive  = obdtoggleswitch.isChecked();
					ready=false;
					return false;
				}
        		
        	});
        }  
        /**
         * Start OBD2_Activity on click, which then calls a dialog for connecting to
         * a bluetooth device an initializing the connection
         * Only available if switch or checkbox is toggled to true
         * @see OBD2_Activity
         */
        Preference obdconnect = (Preference)findPreference("obd2connectpref");
        obdconnect.setOnPreferenceClickListener(new OnPreferenceClickListener(){
        	
        	/**
        	 *Starts {@link OBD2_Activity} on click 
        	 */
			public boolean onPreferenceClick(Preference preference) {
				Intent intent = new Intent(iTloggerPreferences.this, OBD2_Activity.class);
				intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
				startActivity(intent);
				return false;
			}
        });  
        /**
         * Is only enabled when connected to a OBD2 bluetooth device
         * creates a dialog in which the user can choose from available OBD2 sensors
         */
        String obdfirst = getString(R.string.connectobdfirstpref);
        Preference obdsensor = (Preference)findPreference("obd2sensorspref");
        if(ready==true) {
        	obdsensor.setEnabled(true);
        } else{
        	obdsensor.setEnabled(false);
        	obdsensor.setSummary(obdfirst);
        }
        obdsensor.setOnPreferenceClickListener(new OnPreferenceClickListener() {

			public boolean onPreferenceClick(Preference preference) {
				showDialog(sensor);
				return false;
			}
        });
    }
    /**
     * Switches different dialogs, depending on the id it is called with
     * @param id The id of the dialog to be called, int
     */
    @Override
    protected Dialog onCreateDialog(int id) {
        switch (id) {
            case sensor:
            	String vehid = getString(R.string.vehid);
        		String enginerpm = getString(R.string.enginerpm);
        		String vehiclespeed = getString(R.string.vehiclespeed);
        		String throttlepos = getString(R.string.throttlepos);
        		String barpressure = getString(R.string.barpressure);
        		String airtemp = getString(R.string.airtemp);
        		String torque = getString(R.string.torque);
        		String reftorque = getString(R.string.reftorque);
        		String intakeairtemp = getString(R.string.intakeairtemp);
        		String na = getString(R.string.notav);
        		CharSequence[] itemsobd= new CharSequence[9];
        		final boolean[] itemsobdselected= new boolean[9];
        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_EngineRPM-1]==true) {
        			itemsobdselected[0]=OBD2_Activity.PID_all[OBD2_Activity.Value_EngineRPM-1];
        			itemsobd[0]=enginerpm;
        		} else {
        			itemsobdselected[0]=false;
        			itemsobd[0]=enginerpm+": "+na;
        		}
        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_VehicleSpeed-1]==true) {
        			itemsobdselected[1]=OBD2_Activity.PID_all[OBD2_Activity.Value_VehicleSpeed-1];
        			itemsobd[1]=vehiclespeed;
        		} else {
        			itemsobdselected[1]=false;
        			itemsobd[1]=vehiclespeed+": "+na;
        		} 
        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_ThrottlePosition-1]==true) {
        			itemsobdselected[2]=OBD2_Activity.PID_all[OBD2_Activity.Value_ThrottlePosition-1];
        			itemsobd[2]=throttlepos;
        		} else {
        			itemsobdselected[2]=false;
        			itemsobd[2]=throttlepos+": "+na;
        		}
        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_BarometricPressure-1]==true) {
        			itemsobdselected[3]=OBD2_Activity.PID_all[OBD2_Activity.Value_BarometricPressure-1];
        			itemsobd[3]=barpressure;
        		} else {
        			itemsobdselected[3]=false;
        			itemsobd[3]=barpressure+": "+na;
        		}
        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_AmbientAirTemperature-1]==true) {
        			itemsobdselected[4]=OBD2_Activity.PID_all[OBD2_Activity.Value_AmbientAirTemperature-1];
        			itemsobd[4]=airtemp;
        		} else {
        			itemsobdselected[4]=false;
        			itemsobd[4]=airtemp+": "+na;
        		}
        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_ActualEnginePercentTorque-1]==true) {
        			itemsobdselected[5]=OBD2_Activity.PID_all[OBD2_Activity.Value_ActualEnginePercentTorque-1];
        			itemsobd[5]=torque;
        		} else {
        			itemsobdselected[5]=false;
        			itemsobd[5]=torque+": "+na;
        		}
        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_EngineReferenceTorque-1]==true) {
        			itemsobdselected[6]=OBD2_Activity.PID_all[OBD2_Activity.Value_EngineReferenceTorque-1];
        			itemsobd[6]=reftorque;
        		} else {
        			itemsobdselected[6]=false;
        			itemsobd[6]=reftorque+": "+na;
        		}
        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_IntakeAirTemperature-1]==true) {
        			itemsobdselected[7]=OBD2_Activity.PID_all[OBD2_Activity.Value_IntakeAirTemperature-1];
        			itemsobd[7]=intakeairtemp;
        		} else {
        			itemsobdselected[7]=false;
        			itemsobd[7]=intakeairtemp+": "+na;
        		}
        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_VID-1]==true) {
        			itemsobdselected[8]=OBD2_Activity.PID_all[OBD2_Activity.Value_VID-1];
        			itemsobd[8]=vehid;
        		} else {
        			itemsobdselected[8]=false;
        			itemsobd[8]=vehid+": "+na;
        		}
        		String selectsensors = getString(R.string.obd2sensorchoosetext);
            	return new AlertDialog.Builder(iTloggerPreferences.this)
            			.setTitle(selectsensors)
            	       .setMultiChoiceItems (itemsobd, 
            	    		   itemsobdselected, new DialogInterface.OnMultiChoiceClickListener() {
            	    	   
								public void onClick(DialogInterface dialog,
										int which, boolean isChecked) {
									switch (which) {
									case 0:
										if (OBD2_Activity.PID_all[OBD2_Activity.Value_EngineRPM-1]==true) {
										OBD2_Activity.selected[0]=itemsobdselected[0];
										} else {
											OBD2_Activity.selected[0]=false;
										}
										break;
									case 1:
						        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_VehicleSpeed-1]==true) {
										OBD2_Activity.selected[1]=itemsobdselected[1];
						        		} else {
						        			OBD2_Activity.selected[1]=false;
						        		}
										break;
									case 2:
						        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_ThrottlePosition-1]==true) {
										OBD2_Activity.selected[2]=itemsobdselected[2];
						        		} else {
						        			OBD2_Activity.selected[2]=false;
						        		}
										break;
									case 3:
						        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_BarometricPressure-1]==true) {
										OBD2_Activity.selected[3]=itemsobdselected[3];
						        		} else {
						        			OBD2_Activity.selected[3]=false;
						        		}
										break;
									case 4:
						        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_AmbientAirTemperature-1]==true) {
										OBD2_Activity.selected[4]=itemsobdselected[4];
						        		} else {
						        			OBD2_Activity.selected[4]=false;
						        		}
										break;
									case 5:
						        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_ActualEnginePercentTorque-1]==true) {
										OBD2_Activity.selected[5]=itemsobdselected[5];
						        		} else {
						        			OBD2_Activity.selected[5]=false;
						        		}
										break;
									case 6:
						        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_EngineReferenceTorque-1]==true) {
										OBD2_Activity.selected[6]=itemsobdselected[6];
						        		} else {
						        			OBD2_Activity.selected[6]=false;
						        		}
										break;
									case 7: 
						        		if (OBD2_Activity.PID_all[OBD2_Activity.Value_IntakeAirTemperature-1]==true) {
										OBD2_Activity.selected[7]=itemsobdselected[7];
						        		} else {
						        			OBD2_Activity.selected[7]=false;
						        		}
										break;
									case 8:
										OBD2_Activity.selected[8]=itemsobdselected[8];
										break;
									}
								}
            	       })
            	       .setPositiveButton("Ok", new DialogInterface.OnClickListener() {
            	           public void onClick(DialogInterface dialog, int which) {
						}
            	       })
            	       .create();            	
        }
        return null;
    }
    
    /**
     * Make the icon in the top left corner a up-button
	 * @param MenuItem {@link android.R.id.home}: the up-button
	 */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
        	case android.R.id.home:
        		if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.HONEYCOMB_MR2) {
        		 	CheckBoxPreference camtoggle = (CheckBoxPreference)findPreference("camactivepref");
        		 	CheckBoxPreference obdtoggle = (CheckBoxPreference)findPreference("toggleobd2");
    				camactive  = camtoggle.isChecked();
    				obdactive = obdtoggle.isChecked();
        		}else {
        		 	SwitchPreference camtoggleswitch = (SwitchPreference)findPreference("camactivepref");
        		 	SwitchPreference obdtoggleswitch = (SwitchPreference)findPreference("toggleobd2");
    				camactive  = camtoggleswitch.isChecked();
    				obdactive = obdtoggleswitch.isChecked();
        		}
        		// app icon in action bar clicked; go home
        		Intent intent = new Intent(this, iTloggerActivity.class);
        		intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        		startActivity(intent);
        		break;
        }
        return true;
		}
    @Override
    public void onBackPressed() {
    	super.onBackPressed();
    	 if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.HONEYCOMB_MR2) {
    		 	CheckBoxPreference camtoggle = (CheckBoxPreference)findPreference("camactivepref");
    		 	CheckBoxPreference obdtoggle = (CheckBoxPreference)findPreference("toggleobd2");
				camactive  = camtoggle.isChecked();
				obdactive = obdtoggle.isChecked();
    	 }else {
    		 	SwitchPreference camtoggleswitch = (SwitchPreference)findPreference("camactivepref");
    		 	SwitchPreference obdtoggleswitch = (SwitchPreference)findPreference("toggleobd2");
				camactive  = camtoggleswitch.isChecked();
				obdactive = obdtoggleswitch.isChecked();
    	 }
    	Intent intent = new Intent(this, iTloggerActivity.class);
		intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		startActivity(intent);
    }
}